# import olap.xmla.xmla as xmla
# import time
# import simplejson
# import types
# from suds import WebFault
import ujson
# import types
# import pytz
# from types import *
# import re
# import string
from datetime import  datetime, timedelta#,date

from pytz import timezone as timezonepytz
# import time
# import sys,traceback
from django.utils.datastructures import SortedDict
# from django_tools.middlewares import ThreadLocal



# import collections
# import ordereddict
from ordereddict import OrderedDict


from sws_tags.sws_utils.messages import *
from sws_tags.sws_utils.cube_utils import *
from sws_tags.sws_utils.date import *
from sws_tags.sws_utils.__init__ import *


def simboloYvalorTZ(date):
    ##extrae de objeto del tipo datetime el valor y signo de la franja horaria.
    date_str = str(date)

    try:
        if (date_str.find('+')>-1):
            tz=date_str[date_str.find('+')+1:]
            tz=int(tz[:tz.find(':')])
            simbolo='+'
        else:
            tz=date_str[date_str.find(' ')+1:]
            tz=tz[tz.find('-')+1:]
            tz=int(tz[:tz.find(':')])
            simbolo='-'
    except:
        simbolo='+'
        tz=0
    return simbolo,tz

def compruebaFranjaHoraria2(dates,actual_tz_request,time_zone):
    
    madrid = timezonepytz(time_zone)
 
    ##Obtenemos la franja horario de cada fecha para el time zone del settings , por separado simbolo y valor.
    from_date=madrid.localize(datetime(int(dates[0][0:4]), int(dates[0][4:6]), int(dates[0][6:8]),int(dates[0][8:10]),int(dates[0][10:12]) ))    
    from_simbolo,from_tz=simboloYvalorTZ(from_date)
   
    to_date=madrid.localize(datetime(int(dates[1][0:4]), int(dates[1][4:6]), int(dates[1][6:8]),int(dates[1][8:10]),int(dates[1][10:12]) ))
    to_simbolo,to_tz=simboloYvalorTZ(to_date)
    
    actual_date= madrid.localize(datetime.now())
    actual_simbolo,actual_tz=simboloYvalorTZ(actual_date) 
   
    ##Con los valoes obtenidos anteriormente vemos si la fecha consultada tiene una franja horaria diferente a la actual, si es asi procedemos a cambiar las fechas
    if (actual_tz!=to_tz):
        if (to_tz==from_tz):

            diferencia=(int(actual_simbolo+str(actual_tz))-int(from_simbolo+str(from_tz)))*(-1)
            
            from_date=from_date-timedelta(hours=diferencia)
            to_date=to_date-timedelta(hours=diferencia)
                        
    ##Ponemos estas fechas que hemos conseguido en formato string solo con numeros.
    string_date_to=str(to_date)
    string_date_to=string_date_to[:string_date_to.find('+')]
    string_date_to=string_date_to.replace(' ','').replace('-','').replace(':','')

    string_date_from=str(from_date)
    string_date_from=string_date_from[:string_date_from.find('+')]
    string_date_from=string_date_from.replace(' ','').replace('-','').replace(':','')


    return string_date_from,string_date_to

def preCompruebaFranjaHoraria (part_where,actual_tz,time_zone):
    tiempo={'Time':False,'Date':False}
    for i in part_where:
        if i[0]in tiempo:
            tiempo[i[0]]=i
            dim_time=i
    
    aux_fecha0,aux_fecha1=compruebaFranjaHoraria2(part_where[dim_time],actual_tz,time_zone)

    part_where_aux_fechas = {}
    for i in part_where:
        if i == dim_time:
            part_where_aux_fechas[i]=(aux_fecha0,aux_fecha1)
        else:
            part_where_aux_fechas[i]=part_where[i]

    part_where=part_where_aux_fechas
    
    return part_where

def modifyNameBddCubes(name_bbdd,request_data):
    
    if name_bbdd[0]=='ing' or name_bbdd[0]=='sultan':
        
        yearFrom = request_data['from_calldate'].strftime("%Y-%m-%d %H:%M:%S")[0:4]
        yearTo = request_data['to_calldate'].strftime("%Y-%m-%d %H:%M:%S")[0:4]

        monthFrom = request_data['from_calldate'].strftime("%Y-%m-%d %H:%M:%S")[5:7]
        # monthTo = request_data['to_calldate'].strftime("%Y-%m-%d %H:%M:%S")[5:7]
        
        if (yearTo==yearFrom):
            year=yearFrom
        else:
            if int(monthFrom)>=10 :
                year=yearTo
            else:
                year=0
        
        for i in range(len(name_bbdd)):
           
            name_bbdd[i]=name_bbdd[i]+str(year)
    # print '--++ '*100
    # print name_bbdd
    return name_bbdd

def reorderColCube(col_cube,data_order):
    col_cube_aux=[]
    if col_cube:
        if data_order in col_cube:
            col_cube.remove(data_order)
            col_cube=sorted(list(set(col_cube)))
            col_cube_aux.append(data_order)
            for i in col_cube:
                col_cube_aux.append(i)

            return col_cube_aux
        else:
            return col_cube
    return col_cube


def add_MDX_redis(ide,value,r,seconds):            
            
    return r.set(ide+'_MDXS',value,seconds)
    
def get_MDXS_redis(ide,r):

    value = eval(r.get(ide+'_MDXS'))
    
    return value
def is_MDXS_Added_redis(ide,r):                                                                                    
    return not (r.get(ide+'_MDXS') == None)

def segundosHastaY5oEnPunto(minute,seconds):
    if len(str(minute))==1:
        aux=str(minute).zfill(2)
    else:
        aux=str(minute)

    digit1=int(aux[1])
    digit0=int(aux[0])

    if digit1<5:
        digit1=5
    else:
        digit1=0
        digit0=digit0+1

    return ((int(str(digit0)+str(digit1))-minute)*60)-seconds





def requestCube(ip_connect,name_bbdd,name_cube,col_cube,med_cube,request_data,ifDate,dimension_filters,database_cube_dict,total_data = False,where_tupla=('Date','Id','Id'),request_date_previous = False,complete_name=False,format_cube=False,n_c=[],time_zone="",redis=""):
    

    # name_carrier=''
    if 'cube_config' in request_data :
        cube_config=request_data['cube_config']
        ip_connect=cube_config['cube_ip']
        name_bbdd=[cube_config['cube_db']]
        name_cube=request_data['cube_name']
        # name_carrier= name_bbdd[0][name_bbdd[0].index('carrier_')+8:]+'_'
        
        
    
    cube=CUBE()
    
    mdx_class=MDX()
    
    # name_bbdd=modifyNameBddCubes(name_bbdd,request_data)    
    
    
    cube.connect(ip_connect,name_bbdd,n_c,redis) 
    exclude_rows={}
    where={}
    data_order = ''
    part_order={}
    mdx=''
    
    
    part_where ={}
    part_where = filterWhere(dimension_filters,request_data,database_cube_dict)
    for i in part_where:
        where[i]=part_where[i]

    
    if ifDate==True:
       
        if request_date_previous == False:
            where[where_tupla]=(formatDateCube(request_data ['from_Date']),formatDateCube(request_data ['to_Date']))
        else:
            where[where_tupla]=(formatDateCube(request_data ['from_Date_previous']),formatDateCube(request_data ['to_Date_previous']))          
    
   
    sord = request_data['sord'].upper()
   
    sord = 'B'+sord
    for i in col_cube:
        if i[0] == 'Date':
            dato=i[1]
        else:
            dato=i[0]
        if request_data['sidx'] == dato:
            data_order= i

    for i in med_cube:
        if request_data['sidx'] == i[1]:
            data_order= i
            

    col_cube = namesColCube(col_cube)

    data_order = namesColCube(data_order)

    
    part_order[data_order]=sord
    
    # print 'where antes',where
    # where=preCompruebaFranjaHoraria(where,request_data['django_timezone'],time_zone)
    # print 'where despues', where    
    

    col_cube=reorderColCube(col_cube,data_order)


    key=name_bbdd[0]+'MDXS_'+str(col_cube)+str(exclude_rows)+str(med_cube)+str(name_cube)+str(where)+str(part_order)+str(name_bbdd)+str(ip_connect)+str(format_cube)

    
    minute=datetime.now().minute
    second=datetime.now().second
    seconds=segundosHastaY5oEnPunto(minute,second)
    
    if (is_MDXS_Added_redis(name_bbdd[0],redis)):
        MDXS=get_MDXS_redis(name_bbdd[0],redis)
    else:
        MDXS={}



    if not (key in  MDXS.keys()):
        
        mdx=mdx_class.mdx(cube,part_rows=col_cube,exclude_rows=exclude_rows,part_columns=med_cube,part_from=name_cube,part_where=where,part_order=part_order) #ASC or DESC  ('measure','calls')
        
        # print 'MDX',mdx
        res=cube.launch_query(mdx)
        # print ' '*30
        # print res
        
        grid=Salida_Grid()
      
        if format_cube:
        
            json_data=grid.result_to_json(cube,res,database_cube_dict,False,request_data ['row_num'],request_data['page'],total_data,complete_name,format_cube,col_cube)
        
            mdx_total='select ([Client].[Client]) on rows,'+mdx_class.without_rows(mdx)
            
            
            res=cube.launch_query(mdx_total)

          
            json_data_footer=grid.result_to_json(cube,res,database_cube_dict,False,request_data ['row_num'],request_data['page'],total_data,complete_name,format_cube)

            json_data=json.loads(json_data)

            if json.loads(json_data_footer)['rows']:
                json_data['footerData']=json.loads(json_data_footer)['rows'][0]

                try:
                    json_data['footerData'][request_data['group_by']] = 'ALL'
                except:
                    pass


                try:
                    json_data['footerData'][request_data['and_by']] = 'ALL'
                except:
                    pass


            json_data=json.dumps(json_data)

        else:
        
            json_data=grid.result_to_json(cube,res,database_cube_dict,False,request_data ['row_num'],request_data['page'],total_data,complete_name,format_cube,col_cube)
        
       
        if data_order:

            if data_order in col_cube:
        
                if 'ASC' in sord:
                    json_data= ujson.loads(json_data)
                    json_data=reverseJson(json_data)                    
                    json_data=ujson.dumps(json_data)
        MDXS[key]=ujson.loads(json_data)
        add_MDX_redis(name_bbdd[0],MDXS,redis,seconds)
        

    else:
        
        json_data=MDXS[key]
        json_data=ujson.dumps(json_data)

  
    return json_data
def reverseJson(json_data):
    rows=json_data['rows']
    rows.reverse()
    json_data['rows']=rows
    return json_data
def reorderJson(json_data,col_cube,data_order):
    dim1=col_cube[0][0]+'*'+col_cube[0][1]
    dim2=col_cube[1][0]+'*'+col_cube[1][1]

    # rows=[]
   
    for row in json_data['rows']:
        value_dim1=row[dim1]
        value_dim2=row[dim2]
        row[dim1]=value_dim2
        row[dim2]=value_dim1

    if len(col_cube)>2:
        if data_order==col_cube[2]:
            dim1=col_cube[1][0]+'*'+col_cube[1][1]
            dim2=col_cube[2][0]+'*'+col_cube[2][1]

            # rows=[]
           
            for row in json_data['rows']:
                value_dim1=row[dim1]
                value_dim2=row[dim2]
                row[dim1]=value_dim2
                row[dim2]=value_dim1        

  
    return json_data

def requestCubeHighcharts(ip_connect,name_bbdd,name_cube,col_cube,med_cube,request_data,ifDate,dimension_filters,database_cube_dict,total_data = True,where_tupla=('Date','Id','Id'),request_date_previous = False, format_cube=False,n_c=[],time_zone="",redis=""):
    
    # name_carrier=''
    if 'cube_config' in request_data :
        cube_config=request_data['cube_config']
        ip_connect=cube_config['cube_ip']
        name_bbdd=[cube_config['cube_db']]
        name_cube=request_data['cube_name']
        # name_carrier= name_bbdd[0][name_bbdd[0].index('carrier_')+8:]+'_'
  
    cube=CUBE()
    mdx_class=MDX()
  
    # name_bbdd=modifyNameBddCubes(name_bbdd,request_data)    
    
    
    
    cube.connect(ip_connect,name_bbdd,n_c,redis) 
    exclude_rows={}
    where={}
    part_where = filterWhere(dimension_filters,request_data,database_cube_dict)

    for i in part_where:
        where[i]=part_where[i]
    
    if ifDate==True:
        if request_date_previous == False:
            where[where_tupla]=(formatDateCube(request_data['from_Date']),formatDateCube(request_data['to_Date']))
        else:
            where[where_tupla]=(formatDateCube(request_data['from_Date_previous']),formatDateCube(request_data['to_Date_previous']))

 
    new_col_cube=[]
    
    
    for i in col_cube:
        new_col_cube.append((i[0],i[1],i[2]))
    col_cube=new_col_cube
    
       
  
    
    request_data['int_all_time']=0

    
    key=name_bbdd[0]+'MDXS_'+str(col_cube)+str(exclude_rows)+str(med_cube)+str(name_cube)+str(where)+str(name_bbdd)+str(ip_connect)+str(request_data['button'])

    minute=datetime.now().minute
    second=datetime.now().second
    seconds=segundosHastaY5oEnPunto(minute,second)
    
    if (is_MDXS_Added_redis(name_bbdd[0],redis)):
        MDXS=get_MDXS_redis(name_bbdd[0],redis)
    else:
        MDXS={}

    if not (key in  MDXS.keys()):
        
    
        mdx=mdx_class.mdx(cube,part_rows=col_cube,exclude_rows=exclude_rows,part_columns=med_cube,part_from=name_cube,part_where=where) #ASC or DESC  ('measure','calls')

        # print 'MDX HIGHCHARTS',mdx
        
        # print ' '*30
        
        res=cube.launch_query(mdx)
       
        high=Salida_Highcharts()

        types = request_data['representation_type']

        if request_data['button']=='ALL':
            json_data=high.result_to_json(cube,res,types,dimension_v_name='Client',format_cube= format_cube)
        else:
            json_data=high.result_to_json(cube,res,types,dimension_v_name=request_data['button'],exclude=exclude_rows,format_cube= format_cube)

        MDXS[key]=json_data
        add_MDX_redis(name_bbdd[0],MDXS,redis,seconds)

    else:
        
        json_data=MDXS[key]
        

    return json_data

def requestCubeFilters(ip_connect,name_bbdd,name_cube,col_cube,med_cube,request_data,ifDate,dimension_filters,database_cube_dict=False,where_tupla=('Date','Id','Id'),n_c=[],time_zone="",redis=""):
    
    # name_carrier=''
    if 'cube_config' in request_data :
        cube_config=request_data['cube_config']
        ip_connect=cube_config['cube_ip']
        name_bbdd=[cube_config['cube_db']]
        name_cube=request_data['cube_name']
        # name_carrier= name_bbdd[0][name_bbdd[0].index('carrier_')+8:]+'_'

    cube=CUBE()
    mdx_class=MDX()
   
    # name_bbdd=modifyNameBddCubes(name_bbdd,request_data)    
    
  
    
    cube.connect(ip_connect,name_bbdd,n_c,redis) 
    
    where = filterWhere(dimension_filters,request_data,database_cube_dict)
    
    exclude_rows={}
   
    if request_data[request_data['types']]!='NULL':

        
        
        if request_data['types']=='Hangupcode2':
            key=('Hangupcode2','Long Description','Long Description')
        elif request_data['types']=='Hangupcode':
            key=('Hangupcode','Description','Description')
        else:
            key=(str(request_data['types']),'Id','Id')
        col_cube.append(key)
        
       
        exclude_rows[key]=where[key].replace('+',' ')
        
        where_aux={}
        for i in where:
            if i!=key:
                where_aux[i]=where[i]
        where=where_aux
        exclude_rows=[exclude_rows]

    else:
        exclude_rows={}

    if ifDate==True:
        where[('Date','Id','Id')]=(formatDateCube(request_data ['from_Date']),formatDateCube(request_data ['to_Date']))
   
    # where=preCompruebaFranjaHoraria(where,request_data['django_timezone'],time_zone)

    
    key=name_bbdd[0]+'MDXS_'+str(col_cube)+str(exclude_rows)+str(med_cube)+str(name_cube)+str(where)+str(name_bbdd)+str(ip_connect)+str(dimension_filters)

    minute=datetime.now().minute
    second=datetime.now().second
    seconds=segundosHastaY5oEnPunto(minute,second)
    
    # if (is_MDXS_Added_redis(name_bbdd[0],redis)):
    #     MDXS=get_MDXS_redis(name_bbdd[0],redis)
    # else:
    #     MDXS={}

    MDXS={}

    if not (key in  MDXS.keys()):
        
    
        mdx=mdx_class.mdx(cube,part_rows=col_cube,exclude_rows=exclude_rows,part_columns=med_cube,part_from=name_cube,part_where=where)
        


        grid=Salida_Grid()
        
        dimension_filter=filterDimensionCube(dimension_filters,request_data['types'])
        
        if dimension_filter[0][0].find('_')>0:  # especial case two atributes same dimension, delete charts after '_'
            dimension_filter_new = []
            index = dimension_filter[0][0].find('_')
            dimension_filter_new.append((dimension_filter[0][0][0:index],dimension_filter[0][1],dimension_filter[0][2]))
            dimension_filter = dimension_filter_new
        
        if dimension_filter[0][0]=='Hangupcode':
            dimension_filter=[('Hangupcode','Description','Description')]
            filters_cube = grid.filters(mdx,cube,database_cube_dict,dimension_filter,exclude_rows,'None')
        elif dimension_filter[0][0]=='Hangupcode2':
            dimension_filter=[('Hangupcode2','Long Description','Long Description')]
            filters_cube = grid.filters(mdx,cube,database_cube_dict,dimension_filter,exclude_rows,'None')
        else:
            filters_cube = grid.filters(mdx,cube,database_cube_dict,dimension_filter,exclude_rows)

        MDXS[key]=filters_cube
        add_MDX_redis(name_bbdd[0],MDXS,redis,seconds)

    else:
        
        filters_cube=MDXS[key]

    
    json_data = json_encode({
        'more':False,
        'results': filters_cube
        })
 
    return json_data


def filterColCube(col_cube,col_group_by,col_and_by='',col_final_by=''):
    # exclusivas = ['Month Field','Week Field','Day Complete']

    col_cube_new = []
    if (col_group_by!='ALL')&(col_group_by!='All'):
        if (col_group_by!=''):
            added=False
            for c in col_cube:
                if (c[0]==col_group_by or c[2]==col_group_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))             
                    added=True   
    else:
        col_cube_new.append(('Client', 'Client', 'All'))

    if (col_and_by!='ALL')&(col_and_by!='All'):
        if (col_and_by!='' and col_and_by!= col_group_by):
            added=False
            for c in col_cube:
                if (c[0]==col_and_by or c[2]==col_and_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))
                    added=True
    else:
        col_cube_new.append(('Client', 'Client', 'All'))

    if (col_final_by!='ALL')&(col_final_by!='All'):
        if (col_final_by!='' and col_final_by!= col_and_by and col_final_by!= col_group_by):
            added=False
            for c in col_cube:
                if (c[0]==col_final_by or c[2]==col_final_by) and not added:
                    col_cube_new.append((c[0],c[1],c[2]))
                    added=True
    else:
        col_cube_new.append(('Client', 'Client', 'All'))
    return col_cube_new

def namesColCube(col_cube_new):

    if type(col_cube_new) == list:
        col_cube=[]
        for c in col_cube_new:
            index = c[0].find('_')
            if index>0:
                col_cube.append((c[0][0:index],c[1],c[2]))
            else:
                col_cube.append((c[0],c[1],c[2]))
        return col_cube  
    else:
        if len(col_cube_new)>0:
            try:
                index = col_cube_new[0].find('_')
                if index>0:
                    new_name = col_cube_new[0][0:index]
                    col_cube_new=(new_name,col_cube_new[1],col_cube_new[2])
            except Exception, e:
                swslog('error','Index out range',e)
        return col_cube_new


def adjustementData(dict_higcharts,value_adjustement):

    list_index = []
    i=0

    for data in dict_higcharts['v_name']:
        for name in value_adjustement:
            if data.find(name[0]) > 0:
                list_index.append((data,i,name[1]))
                i = i+1

    list_data = []
    list_aux = []

    j = 0
    for data in dict_higcharts['v_data']:

        for i in range(0,len(data)):
    
            try:
        
                list_aux.append(data[i]*list_index[j][2])
            except:
                list_aux.append(data[i])
        j = j+1
        list_data.append(list_aux)
        list_aux = []


    dict_higcharts['v_data'] = list_data

    return dict_higcharts


#return tuple with column and id of the dimemsion selected
def filterDimensionCube(dimension_filters,dimension):
    new_dimension_filters=[]
    for d in dimension_filters:
        if d[0]== dimension:
            new_dimension_filters.append(d[0:3])
    return new_dimension_filters

def filterWhere(filters,request_data,database_cube_dict): 
    
    where={}
 
    for f in filters:
   
        if f[0]!='calldate':
       
            if f[0] in request_data:
                if  request_data[f[0]]!='NULL':
                    if f[0]=='Hangupcode':
                        where[(f[0],'Description','Description')]=request_data[f[0]]
                    elif f[0]=='Hangupcode2':
                        where[(f[0],'Long Description','Long Description')]=request_data[f[0]]
                    else:    
                        where[(f[0],'Id','Id')]=request_data[f[0]]
          
  

    if request_data['param_exclude'] == '1': 

        where[('Hangupcode','Id','Id')]=('-',['34','41'])
        # TODOMAS NO DELETE EXAMPLE USE
        # where[('Hangupcode','Id','Id')]=('-','34')
        # where[('Hangupcode','Id','Id')]=('',['34','41'])  
        # where[('Hangupcode','Id','Id')] = '34'

    return where   

def filterDataCube(filters,request,database_cube_dict): 
    where={}
    for f in filters:
        if f[0]!='calldate':
            if f[0] in request.GET:
                if  request.GET[f[0]]!='NULL':
                    where[(f[0],'id')]=request.GET[f[0]]
    return where

def filterMed(med_cube,select_med):

    if select_med.lower() == 'asr':
        med_cube=[('Measures','ASR'),('Measures','Attempts')]
        return med_cube
    elif select_med.lower() == 'acd' :
        med_cube=[('Measures','ACD'),('Measures','ASR'),('Measures','Attempts')]
        return med_cube
    else:
        return med_cube


def atributeAdjustement(json_data,col_cube,request_data):
    if len(json_data)>70:
        json_data = json_data.replace('Date*','')
        
        #especial case when the query have two atributtes of the same dimension
        if request_data['group_by'].find('_')>0 :
            group_by_plus= request_data['group_by'].replace('_','*')
            json_data = json_data.replace(group_by_plus,request_data['group_by'])

        if 'and_by' in request_data:
            try:
                if request_data['and_by'].find('_')>0:
                    and_by_plus = request_data['and_by'].replace('_','*')
                    json_data = json_data.replace(and_by_plus,request_data['and_by'])
            except Exception, e:
                swslog('error', 'non request_data[and_by]',e)

        for name in col_cube:
            json_data = json_data.replace('*'+name[1]+'"','"')
   
    return json_data


def obtainMargin(json_data):
    json_data_dict = ujson.loads(json_data)
    
    for data in json_data_dict['rows']:
        if float(data['Total Income']) != 0:
            # benefit = float(data['Total Income']) - float(data['Total Cost'])
            # margin = benefit / float(data['Total Income']) * 100
            dif = random.randrange(0,9)
            data['Margin'] = dif
        else:
            data['Margin'] = 0
    json_data = json.dumps(json_data_dict)

    return json_data


def orderDataCube(request,json_data):
    json_data_dict = ujson.loads(json_data)
    name_order = request['sidx']
    if request['sord'] == 'asc':
        rev = False
    else:
        rev = True
    sorted_json = sorted(json_data_dict['rows'], key=lambda d: d[name_order],reverse = rev)
    json_data_dict['rows'] = sorted_json
    json_data_dict = json.dumps(json_data_dict)

    return json_data_dict


def accuratelyAdjustment(json_data,med_cube_prec,dict_type_data = False,request_data=False,format_excel= False):

    # request = ThreadLocal.get_current_request()

    if len(med_cube_prec[0])==3:
        ind = 0
    else:
        ind = 1

    json_data_dict = json.loads(json_data,object_pairs_hook=OrderedDict)

    new_list_data = []
    for i in range(len(json_data_dict['rows'])):
        new_dict = SortedDict()
        for k,v in json_data_dict['rows'][i].items():
            for m in med_cube_prec:
                if m[ind] == k:

                    v = str(v)
                    
                    is_ip = False
                    ini_v = v.find('.')
                    sub_v= v[ini_v+1:]
                    
                    if sub_v.find('.')>=0:
                        is_ip=True

                    if v.find('.')>=1 and v != '0.0.0.0' and is_ip==False:

                        index = v.find('.');
                        if m[ind+1] != 0:
                            v = v[0:index+1+m[ind+1]]
                        else:
                            v = v[0:index]
                        
                        if format_excel == True:
                            v = float(v)
                        else :
                            v = v.replace('.',',')
                            if dict_type_data:
                                if k in dict_type_data and dict_type_data[k]:

                                    try:    
                                        float(v)             
                                        v = numberFormatter(v)
                                    except Exception, e:
                                        pass
                                        # swslog('error','Number formater not float:'+str(type(v))+str(v),e)
                            try:
                                v = v + m[ind+2]
                            except Exception, e:
                                swslog('error','Add type data for measure cube',e)
                # new_dict[k] = v

            new_dict[k] = v
        new_list_data.append(new_dict)
    json_data_dict['rows'] = new_list_data

    try:

        new_dict_footer={}
        if 'footerData' in json_data_dict:
            for k,v in json_data_dict['footerData'].items():
                for m in med_cube_prec:
                    if m[ind] == k:
                        v = str(v)
                        v = v.replace(',','.')
                        if v.find('.')>=1 and v != '0.0.0.0':
                            index = v.find('.');
                            if m[ind+1] != 0:
                                v = v[0:index+1+m[ind+1]]
                            else:
                                v = v[0:index]

                            if dict_type_data:
                                if dict_type_data[k]:
                                    try:
                                        float(v)             
                                        v = numberFormatter(v)
                                    except Exception, e:
                                        swslog('error','Number formater not float:'+str(type(v))+str(v),e)
                        try:
                            v = v + m[ind+2]
                        except Exception, e:
                            # logger.error('ERROR: in add type data for measure cube')
                            # logger.error('Exception: {0}'.format(e))
                            swslog('error','Add type data for measure cube',e)
                    # new_dict[k] = v
        
                new_dict_footer[k] = v
            json_data_dict['footerData'] = new_dict_footer
    except Exception,e:
        swslog('info','json without footerData',e)



    json_data = json.dumps(json_data_dict)  
    return json_data
    # else:

    #     for i in range(0,len(queryset_detail)):
    #         for j in field_query_detail:
    #             data = str(queryset_detail[i][j[0]])
    #             if data.find('.')>0:
    #                 index = data.find('.')
    #                 queryset_detail[i][j[0]] = data[0:index+1+j[1]]+j[2]


# def accuratelyNumber(v):
#     v = v.replace('.',',')
#     index = v.find(',')
#     if index > 0:
#         sub_v = v[0:index]   #parte entera            
#         if len(sub_v)>3: 
#             new_name = ''
#             i = 0
#             for t in range(len(sub_v),0,-1):
#                 t = t-1
#                 new_name = sub_v[t] +new_name 
#                 i = i+1
#                 if (i%3)==0 and i!=0:
#                     new_name =  '.'+new_name 
#             v = new_name+v[index:]
#         if v[0] == '.':
#             v = v[1:]
#     return v


def filterFormat(new_queryset):
    dict_type_data={}
    try:

        if len(new_queryset)>0:
            for k,v in new_queryset[0].items():
                if (type(v) is not unicode) and (type(v) is not datetime): 
                    dict_type_data [k] = type(v)
    except Exception,e:

        swslog('debug','List index out range',e)
        dict_type_data=False

    return dict_type_data   


def numberFormatter(number):
  # Nos aseguramos de que sea un string
  a = str(number)
  # Buscamos la posicion donde esta el separador de decimales
  p = a.find('.')
  # Guardamos el string con los decimales sustituyendo el punto por una coma (si es que hay)
  if p == -1:
    decimales = ''
  else:
    decimales = a[p:].replace('.',',')
    # Cogemos el entero
    a = a[:p]
  # Variables auxiliares
  b = ''
  c = ''
  i = 0
  # Generamos el numero/string con los miles
  while i < len(a)/3:
    b = a[len(a)-(3*(i+1)):len(a)-(len(c)-i)] 
    c = '.'+b + c
    i+=1
  # Concatenamos los decimales al resultado
  c = a[:len(a)-(3*i)] + c + decimales
  # Devolvemos el resultado

  if c[0]=='.':
    c = c[1:]

  c = c.replace('.',',')

  return c






def getTableDict(json_data, title_dict, value_group_by_data):
    # table_dict = SortedDict({'titles_table':title_dict})
    json_data_dict = ujson.loads(json_data)  
    data = json_data_dict['rows']

    first_element = True
    for key,value in title_dict.items():
        if first_element == True:
            title_dict.pop(key)
            title_dict.insert(0,value_group_by_data,value_group_by_data.capitalize())
            first_element = False

    # table_dict = SortedDict({'titles_table':title_dict})
    cont = 0
    series_dict = SortedDict()
    for d in data:
        exists = False
        for key,value in series_dict.items():
            if value_group_by_data == 'ALL':
                value_group_by_data = 'Client'
            if(value[value_group_by_data] == d[value_group_by_data]):
                row_dict = SortedDict()
                for v in value:
                    element = series_dict.get(key)
                    if v == value_group_by_data:
                        element_value = d[v]
                    elif v == 'ASR':
                        # try:
                        #     element_value = float(element[v]) + float(d[v])
                        #     element_value = str(round(element_value,3))
                        # except:
                        element_value = str(float(element[v]) + float(d[v]))

                    elif v == 'ACD':
                        element_value = str(float(element[v]) + float(d[v]))
                    elif v == 'answer_calls':

                        # element_value = element[v]
                        if value['ASR'] == '0.0' or value['Calls'] == '0.0':
                            element_value = '0'
                        else:
                            if float (value['Calls']) != 0:
                                element_value = str(float(value['ASR']) / float(value['Calls'])) 
                            else:
                                element_value = '0';
                    elif v == 'calls':
                        element_value = str(int(element[v]) + int(d[v]))
                    

                    row_dict[v] = element_value
                
                series_dict[serie] = row_dict
                exists = True

        if not(exists):
            cont = cont + 1
            serie = 'serie' + str(cont)

            row_dict = SortedDict()
            for e in title_dict.keys():
                if e == 'answer_calls':
                    row_dict[e] = '0'
                else:
                    if e == 'ALL':
                        e = 'Client'


                    row_dict[e] = d[e]
            series_dict[serie] = row_dict


import olap.xmla.xmla as xmla
import simplejson
import types
from django.utils.dateparse import parse_datetime
import pytz
from pytz import timezone, utc
from suds import WebFault
import memcache
# from international.models import *
import json
import types
from types import *
import re
import string
from datetime import date, datetime, timedelta
import time
from django.conf import settings
from django.utils.datastructures import SortedDict
from django.utils import timezone

from sws_tags.sws_utils.cube_utils import *
from sws_tags.sws_utils.__init__ import *

granularity=[
    ('Time','Minute Field','Minute Field'), # 0
    ('Time','Ten Minutes','Ten Minutes'),   # 1
    ('Time','Hour Half','Hour Half'),       # 2
    ('Time','Hour Field','Hour Field'),     # 3
    ('Time','Day Quarter','Day Quarter'),   # 4
    ('Time','Day Half','Day Half'),         # 5
    ('Date','Day Complete','Day Complete'),       # 6
    ('Date','Week Complete','Week Complete'),     # 7
    ('Date','Month Complete','Month Complete'),   # 8
    ('Date','Year Complete','Year Complete'),      # 9
    ('Date','Date Complete','Date Complete')
    ]

def strToDatetime(date_str):
    date_str = str(date_str)
    year = date_str[0:4]
    month = date_str[5:7]
    day = date_str[8:10]
    hour = date_str[11:13]
    minute = date_str[14:16]
    secs= date_str[17:19]


    if hour == '':
        hour = '00'
    if minute=='':
        minute ='00'
    if secs=='':
        secs = '00'

    date = datetime(int(year),int(month),int(day),int(hour),int(minute),int(secs))
    return date

def dateToArray(d):
    date_array=[]
    date_array.append(int(d[0:4]))
    date_array.append(int(d[4:6]))
    date_array.append(int(d[6:8]))
    date_array.append(int(d[8:10]))
    date_array.append(int(d[10:12]))
    date_array.append(int('00'))
    date_array.append(0)
    date_array.append(0)
    date_array.append(0)
    return date_array

def intToGMT(number):
    number = str(number)
    if number == '0':
        GMT = 'Etc/GMT'
    elif number.find('-')==0:
        GMT = 'Etc/GMT'+number
    else:
        GMT = 'Etc/GMT+'+number
    return GMT    

def intToGMTInvert(number):
    number = str(number)
    if number == '0':
        GMT = 'Etc/GMT'
    elif number.find('-')==0:
        number = number.replace('-','')
        GMT = 'Etc/GMT+'+number
    else:
        GMT = 'Etc/GMT-'+number
    return GMT    





############################################-- updateDateTz--###################################################
#     Request: updateDateTz(request_data)  
#     Description: This function update the values of the dictionary request_data, update values related with datetime
#     Param:
#         'request_data' -> is a dictionary with the value of request.GET and other 
#     Update:
#          request_data['tz_adjustement'] ->  store the diference in hours inter the hour in GMT of the system and hour in GMT of the user
#          request_data ['from_Date_tz']  -> This is date ' from 'of user in GMT
#          request_data ['to_Date_tz']  -> This is date ' to 'of user in GMT
#          request_data ['from_Date_sw'] -> This is date 'from' of user in format AAAA-
#
def updateDateTz(request_data):
    # 

    if request_data['django_timezone'] != '':
        user_tz = request_data['django_timezone'] 
    else:
        user_tz = pytz.timezone(settings.TIME_ZONE)

    fd = strToDatetime(request_data ['from_Date'])
    td  = strToDatetime(request_data['to_Date'])

    from_dsn=fd
    to_dsn=td
    # to_dsn = selected_tz.localize(td)

    if to_dsn.second == 0 and to_dsn.minute == 0 and to_dsn.hour == 0:
        request_data ['from_Date_tz'] = datetime( from_dsn.year,from_dsn.month,from_dsn.day,00,00,00,tzinfo=user_tz)
        request_data ['to_Date_tz'] = datetime( to_dsn.year,to_dsn.month,to_dsn.day,23,59,59,tzinfo=user_tz)
    else:
        request_data ['from_Date_tz'] = datetime( from_dsn.year,from_dsn.month,from_dsn.day,from_dsn.hour,from_dsn.minute,from_dsn.second,tzinfo=user_tz)
        request_data ['to_Date_tz'] = datetime( to_dsn.year,to_dsn.month,to_dsn.day,to_dsn.hour,to_dsn.minute,to_dsn.second,tzinfo=user_tz)
   
  
    # from_dsn = stonework_tz.normalize(request_data ['from_Date_tz'])
    # to_dsn = stonework_tz.normalize(request_data ['to_Date_tz'])
    tz = pytz.timezone(settings.TIME_ZONE)


    request_data ['from_Date_sw'] = tz.normalize(request_data ['from_Date_tz'])
    request_data ['to_Date_sw'] = tz.normalize(request_data ['to_Date_tz'])

    if (request_data['from_Date_sw'].tzinfo != request_data['to_Date_sw'].tzinfo):
        request_data ['from_Date_sw'] = request_data ['from_Date_tz']
        request_data ['to_Date_sw'] = request_data ['to_Date_tz']

def tzAdjustement(v,tz_adjustement):
    date_datetime = strToDatetime(v)
    date_utz = date_datetime - timedelta(hours=tz_adjustement)
    # date_utz = timezone.make_aware(date_datetime, timezone.get_default_timezone())
    return date_utz

    # if tz_adjustement>0:
    #     date_utz = date_datetime + timedelta(hours=tz_adjustement)
    # elif tz_adjustement<0:
    #     date_utz = date_datetime - timedelta(hours=tz_adjustement)
    # else:
    #     date_utz = date_datetime

def correspondGranularity(request_data):

    if (isinstance(request_data['from_Date'],str)) == True:
        request_data['from_Date'] = unicode(request_data['from_Date'])
        request_data['to_Date'] = unicode(request_data['to_Date'])

    if (isinstance(request_data['from_Date'],unicode)) == False: 
        from_date = request_data['from_Date']
        to_date = request_data['to_Date']
    else:

        if (isinstance(request_data['from_Date'],unicode)): 
            year = str(request_data['from_Date'])[0:4]
            month = str(request_data['from_Date'])[5:7]
            day = str(request_data['from_Date'])[8:10]
            hour = str(request_data['from_Date'])[11:13]
            minute = str(request_data['from_Date'])[14:15]
            from_date = datetime(int(year),int(month),int(day),int(hour),int(minute),00)

        if (isinstance(request_data['to_Date'],unicode)): 
            year = str(request_data['to_Date'])[0:4]
            month = str(request_data['to_Date'])[5:7]
            day = str(request_data['to_Date'])[8:10]
            hour = str(request_data['to_Date'])[11:13]
            minute = str(request_data['to_Date'])[14:15]
            to_date = datetime(int(year),int(month),int(day),int(hour),int(minute),00)
    from_seg=(time.mktime(from_date.timetuple()))
    to_seg=(time.mktime(to_date.timetuple()))
    hour = int((to_seg - from_seg)/3600)

    if len(str((request_data['from_Date'])))==16:
        base_date = str(request_data['from_Date'])+':00'
    else:
       base_date = str(request_data['from_Date']) 

    granularity_index = 0

    if hour <= 1: 
        granularity_index = 0 #Minute Field 
        request_data['base_Date'] = base_date[:-11]

    elif hour <= 3:
        granularity_index = 1 #Ten Minutes
        request_data['base_Date'] = base_date[:-11]


    elif hour <= 12:
        granularity_index = 2 # Hour Half
        request_data['base_Date'] = base_date[:-11]

    elif hour <= 24: 
        granularity_index = 3 # Hour Field 
        request_data['base_Date'] = base_date[:-11]

    elif hour <= 72:
        granularity_index = 4 # Day quarter
        request_data['base_Date'] = base_date[:-11]

    elif hour <= 168:
        granularity_index = 5 # Day half 
        request_data['base_Date'] = base_date[:-11]

    elif hour <= 744:
        granularity_index = 6 # Day Field
        request_data['base_Date'] = base_date[:-11]

    elif hour <=4464:
        granularity_index = 7 # Week Field
        request_data['base_Date'] = base_date[:-14]

    elif hour <=8760:
        granularity_index = 8 # Month Field 
        request_data['base_Date'] = base_date[:-14] 

    else:
        granularity_index = 9 # Year Field
        request_data['base_Date'] = base_date[:-17]



    if granularity_index == 5:
        granularity_index =4 

    # if request_data ['int_compare'] == 'ON':   for first load higstoc by all data
    #     granularity_index = 6

    return granularity_index


def numberHoursByGranularity(granularity):

    
    if granularity ==0:
        hour=1
    elif granularity ==1:
        hour = 1
        # hour = 1
    elif granularity == 2:
        hour = 3
        # hour = 12
    elif granularity == 3:
        # hour = 12
        hour =24
    elif granularity == 4:
        # hour = 24
        # hour = 72
        hour = 168
    elif granularity == 5:
        # hour = 72
        hour = 168
    elif granularity == 6:
        # hour = 168
        hour = 744
    elif granularity == 7:
        hour = 744
        # hour = 4464
    elif granularity == 8:
        hour = 4464
        # hour = 8760
    else: 
        hour = 8760
        # hour = 999999

    return hour


def adjustementGranularity(request_data,col_cube): 

    if request_data['agrupation_data'].find('-')>=0:
        granularity_adjustement = int(request_data['agrupation_data'].replace('-',''))*-1     
    else:
        granularity_adjustement = int(request_data['agrupation_data'].replace('-',''))
    granularity_index = correspondGranularity(request_data)
   
    if request_data['int_compare'] == 'ON':
        if granularity_adjustement > 2:
            granularity_adjustement = 2
        if granularity_adjustement < -2:
            granularity_adjustement = -2

    
    index = granularity_index + granularity_adjustement

    
    if index < 0 :
        index = 0
    index_col = index
    if index_col>7:
        index_col = 7
    # if index_col <1:
    #     index_col = 1



    if index_col < 6:
        col_cube.append(granularity[6]) #add column date field 

    
    # #TODOMAS
    # if index_col > 5:
    #     col_cube.append(granularity[10])
    #     request_data['base_Date'] = ''
    # else:
    
    col_cube.append(granularity[index_col])

    request_data['granularity_text'] = granularity[index_col][2].replace('Field','')
    request_data['granularity_text'] = request_data['granularity_text'].replace('Complete','') 

    if request_data['granularity_text'] == 'Hour Half':
        request_data['granularity_text'] = 'Half Hour'
    elif request_data['granularity_text'] == 'Day Half':
        request_data['granularity_text'] = 'Half Day'


    request_data ['granularity_index'] = index

    ##Calculate date previous

    if (isinstance(request_data['from_Date'],unicode)) == False: 
        from_date = request_data['from_Date']
        to_date = request_data['to_Date']
    else:

        if (isinstance(request_data['from_Date'],unicode)): 
            year = str(request_data['from_Date'])[0:4]
            month = str(request_data['from_Date'])[5:7]
            day = str(request_data['from_Date'])[8:10]
            hour = str(request_data['from_Date'])[11:13]
            minute = str(request_data['from_Date'])[14:16]
            request_data['from_Date'] = datetime(int(year),int(month),int(day),int(hour),int(minute),00)

        if (isinstance(request_data['to_Date'],unicode)): 
            year = str(request_data['to_Date'])[0:4]
            month = str(request_data['to_Date'])[5:7]
            day = str(request_data['to_Date'])[8:10]
            hour = str(request_data['to_Date'])[11:13]
            minute = str(request_data['to_Date'])[14:16]
            request_data['to_Date'] = datetime(int(year),int(month),int(day),int(hour),int(minute),00)

    if request_data['int_forecast'] == 'OFF':
        forecast = 0
    else:
        forecast = 1

    if granularity_index == 0 and forecast==1:
        granularity_index = 1

    elif granularity_index == 1 and forecast==1:
        granularity_index = 2

    
    hours_granularity = numberHoursByGranularity(granularity_index+forecast)

    request_data['dif_date'] = timedelta(hours=hours_granularity)
    
    request_data['from_Date_previous'] = request_data['from_Date'] - request_data['dif_date']

    
    request_data['to_Date_previous'] = request_data['from_Date_previous']  + request_data['dif_date'] - timedelta(seconds = 1)

    

def formatDateCubeComplete(date):
    date=str(date).replace(' ','T')
    return date

def formatDateCube(date):
    date=str(date).split('+')[0].replace('-','').replace(' ','').replace(':','')
    len_date = len(date)
    if len_date==14:
        date = date[0:-2]
    while len(date)<14:
        date = date + '0'
    return date

def formatDateCubeDate(date):
    date=str(date).replace('-','').replace(' ','').replace(':','')
    len_date = len(date)
    if len_date==14:
        date = date[0:len_date-2]
    while len(date)<14:
        date = date + '0'


    if date[8:12] == '2359':
        date_int = int(date[0:8])+1
        date = str(date_int)+'000000'
    return date


def selectGranularity(request_data,col_cube):
    dif_date = int(formatDateCube(request_data ['to_Date'])) - int(formatDateCube(request_data ['from_Date']))
    range_date = ''

    if dif_date <= 10000:
        range_date = 'ten minutes'
        col_cube.append(('timedimension',range_date))

    elif dif_date <= 30000:
        range_date= 'hour half'
        col_cube.append(('timedimension',range_date))
        
    elif dif_date <= 240000:
        range_date= 'hour'
        col_cube.append(('timedimension',range_date))
        
    # elif dif_date <= 1680000:
    #     range_date= 'fecha'
    #     col_cube.append(('cdrfint',range_date))
        
    else:
        range_date= 'fecha'
        col_cube.append(('cdrfint',range_date))




def selectDatePrevious(request_data):

    from_seg=(time.mktime(from_date.timetuple()))
    to_seg=(time.mktime(to_date.timetuple()))
    hour = int((to_seg - from_seg)/3600)

    request_data['to_Date'] = str(request_data['to_Date']) 
    request_data['from_Date'] = str(request_data['from_Date']) 

    
    if dif_date <= 10000: #'hour'
        request_data['dif_date']  = 1
        hour = str(int(request_data['from_Date'][11:13]) - 1)
        if len(hour)==1:
            hour = '0'+hour
        new_hour = request_data['from_Date'][0:11] + hour + request_data['from_Date'][13:]
        request_data['from_Date_previous'] = new_hour

        hour = str(int(request_data['to_Date'][11:13]) - 1)
        if len(hour)==1:
            hour = '0'+ hour
        new_hour = request_data['to_Date'][0:11] + hour + request_data['to_Date'][13:]
        request_data['to_Date_previous'] = new_hour


    elif dif_date <= 30000:
        hour = str(int(request_data['from_Date'][11:13]) - 3)
        if len(hour)==1:
            hour = '0'+hour
        new_hour = request_data['from_Date'][0:11] + hour + request_data['from_Date'][13:]
        request_data['from_Date_previous'] = new_hour

        hour = str(int(request_data['to_Date'][11:13]) - 3)
        if len(hour)==1:
            hour = '0'+ hour
        new_hour = request_data['to_Date'][0:11] + hour + request_data['to_Date'][13:]
        request_data['to_Date_previous'] = new_hour
        request_data['dif_date']  = 3
        
    elif dif_date <= 1000000:
        day = str(int(request_data['from_Date'][8:10]) - 1)
        if len(day)==1:
            day = '0'+day
        new_date = request_data['from_Date'][0:8] + day + request_data['from_Date'][10:]
        request_data['from_Date_previous'] = new_date

        day = str(int(request_data['to_Date'][8:10]) - 1)
        if len(day)==1:
            day = '0'+day
        new_date = request_data['to_Date'][0:8] + day + request_data['to_Date'][10:]
        request_data['to_Date_previous'] = new_date
        request_data['dif_date']  = 24

    elif dif_date <= 6230000 :

        day = str(int(request_data['from_Date'][8:10]) - 7)
        if len(day)==1:
            day = '0'+day
        new_date = request_data['from_Date'][0:8] + day + request_data['from_Date'][10:]
        request_data['from_Date_previous'] = new_date

        day = str(int(request_data['to_Date'][8:10]) - 7)
        if len(day)==1:
            day = '0'+day
        new_date = request_data['to_Date'][0:8] + day + request_data['to_Date'][10:]
        request_data['to_Date_previous'] = new_date
        request_data['dif_date']  = 168
        
    else:
        if int(request_data['from_Date'][5:7]) ==1:
            anio = str(int(request_data['from_Date'][:4]) - 1)
            new_date = anio + '-12-'+request_data['from_Date'][8:]
            request_data['from_Date_previous'] = new_date

            anio = str(int(request_data['to_Date'][:4]) - 1)
            new_date = anio + '-12-'+request_data['to_Date'][8:]
            request_data['to_Date_previous'] = new_date
            request_data['dif_date']  = 5040
        else:
            mes = str(int(request_data['from_Date'][5:7]) - 1)
            if len(mes)==1:
                mes = '0'+mes
            new_date = request_data['from_Date'][0:5] + mes + request_data['from_Date'][7:]
            request_data['from_Date_previous'] = new_date

            mes = str(int(request_data['to_Date'][5:7]) - 1)
            if len(mes)==1:
                mes = '0'+mes
            new_date = request_data['to_Date'][0:5] + mes + request_data['to_Date'][7:]
            request_data['to_Date_previous'] = new_date
            request_data['dif_date']  = 5040



def getFirstWeekDay(start_date):
    current_weekday = start_date.weekday()
    first_weekday = start_date - timedelta(days=current_weekday)
    # first_weekday = datetime(first_weekday.year, first_weekday.month, first_weekday.day)
    return first_weekday










# def updateDateTz(request_data):

#     # try:    
#     #     tz_adjustement = int(request_data['user_tz']) + int(request_data ['system_tz'])
#     # except:
#     #     tz_adjustement = int(request_data ['system_tz'])

#     # request_data['tz_adjustement'] = tz_adjustement

#     # ########stoneworktz
#     # tz_utc = (datetime.now() - datetime.utcnow())+ timedelta(seconds=1)  

#     # time_zone_sw = intToGMT(tz_utc.seconds/3600)

#     # stonework_tz = pytz.timezone(time_zone_sw)  
#     # stonework = stonework_tz.localize(datetime.now())
#     # ######################

#     # time_zone_user = intToGMT(tz_adjustement)
    
#     gap_system = request_data['system_tz'].replace('Etc/GMT','')
#     gap_user = request_data['user_tz'].replace('Etc/GMT','')
#     # request_data['tz_adjustement'] = int(gap_system) - int(gap_user)
#     request_data['tz_adjustement'] = gap_user

#     selected_tz = pytz.timezone(request_data['user_tz'])
#     stonework_tz = pytz.timezone(request_data['system_tz'])  

#     fd = strToDatetime(request_data ['from_Date'])
#     td  = strToDatetime(request_data['to_Date'])



#     # tz_adjustement = int(request_data['tz_adjustement'])

#     # if tz_adjustement>0:
#     #     fd = fd - timedelta(hours=tz_adjustement)
#     # elif tz_adjustement<0:
#     #     fd = fd + timedelta(hours=tz_adjustement)
#     # else:
#     #     fd = fd


#     # if tz_adjustement>0:
#     #     td = td - timedelta(hours=tz_adjustement)
#     # elif tz_adjustement<0:
#     #     td = td + timedelta(hours=tz_adjustement)
#     # else:
#     #     td = td
#     # 
#     from_dsn = selected_tz.localize(fd)
#     to_dsn = selected_tz.localize(td)

#     # from_dsn = selected_tz.normalize(stonework.astimezone(from_date_select))
#     # to_dsn = selected_tz.normalize(stonework.astimezone(to_date_select))

#     # request_data ['from_Date_user'] = request_data ['from_Date']
#     # request_data ['to_Date_user'] =  request_data ['to_Date']

#     request_data ['from_Date_tz'] = datetime( from_dsn.year,from_dsn.month,from_dsn.day,from_dsn.hour,from_dsn.minute,from_dsn.second,tzinfo=selected_tz)
#     request_data ['to_Date_tz'] = datetime( to_dsn.year,to_dsn.month,to_dsn.day,to_dsn.hour,to_dsn.minute,to_dsn.second,tzinfo=selected_tz)


#     # 
#     # 